from math import ceil

from paginator.consts import (
    BACKWARD_LABEL,
    CURRENT_LABEL,
    FORWARD_LABEL
)


def mark_if_current_page(current_page: int,
                         page_number: int) -> str:
    """Adds label to the left of a page number"""
    return str(CURRENT_LABEL) + str(page_number) \
        if page_number == current_page else \
        str(page_number)


def compose_first_line(current_page: int,
                       pages_per_line: int,
                       total_pages: int) -> list[str]:
    start = []
    middle = [
        mark_if_current_page(current_page, page)
        for page in range(1, min(pages_per_line, total_pages) + 1)
    ]
    end = [] if total_pages <= pages_per_line \
        else str(pages_per_line + 1) + str(FORWARD_LABEL)

    return start, middle, end


def compose_last_line(current_line: int,
                      total_pages: int,
                      current_page: int,
                      pages_per_line: int) -> list[str]:

    start = str(BACKWARD_LABEL) + \
            str(current_line * pages_per_line - pages_per_line)
    middle = [
        mark_if_current_page(current_page, page)
        for page in range(
            current_line * pages_per_line - pages_per_line + 1,
            total_pages + 1
        )
    ]

    end = []

    return start, middle, end


def compose_middle_line(current_line: int,
                        current_page: int,
                        pages_per_line: int) -> list[str]:

    start = str(BACKWARD_LABEL) + \
            str(current_line * pages_per_line - pages_per_line)
    middle = [
        mark_if_current_page(current_page, page)
        for page in range(
            current_line * pages_per_line - pages_per_line + 1,
            current_line * pages_per_line + 1
        )
    ]
    end = str(pages_per_line * current_line + 1) + str(FORWARD_LABEL)

    return start, middle, end


def build(total_pages: int,
          total_lines: int,
          current_page: int,
          pages_per_line: int) -> list[str]:
    """
    Builds a markup map as a list of str

    Example of a returned li    st:
    [1] is a page
    [1] [2] [3] - is a line
    [<<4] and [7>>] - are decoration pages

          [V1] [2] [3] [4>>]      - first line example
    [<<4] [5V] [6] [7]            - last line example
    [<<3] [4V] [5] [6]   [7>>]    -  middle line example
    The current page marked as V and may be any page from a current
    line with the exception of decoration pages ([<<4] and [7>>])
    """

    result = []

    current_line = ceil(current_page / pages_per_line)

    if current_line == 1:

        start, middle, end = compose_first_line(
            current_page=current_page,
            pages_per_line=pages_per_line,
            total_pages=total_pages
        )

    elif current_line == total_lines:

        start, middle, end = compose_last_line(
            current_line=current_line,
            total_pages=total_pages,
            current_page=current_page,
            pages_per_line=pages_per_line
        )
    else:  # middle line

        start, middle, end = compose_middle_line(
            current_line=current_line,
            current_page=current_page,
            pages_per_line=min(pages_per_line, total_pages)
        )

    result.append(start) if start else result
    result.extend(middle)
    result.append(end) if end else result

    return result



from telegram import InlineKeyboardButton

from paginator.consts import (
    BACKWARD_LABEL, FORWARD_LABEL
)


def compose_tg_menu(navigation_mockup: list[str],
                    callback_signature_char: str,
                    page_label: str) -> list[list[InlineKeyboardButton]]:
    """
    Creates a list of a list of InlineKeyboardButton

    Args:
        navigation_mockup (list[str]): eg: ['<<2', '4', '5V', '6', '7>>']  # noqa
        callback_signature_char: str
        page_label: str
    """

    menu = []

    # Create a telegram inline buttons structure for a render
    for idx, button in enumerate(navigation_mockup):

        # Extract number
        page_number = int(''.join(filter(str.isdigit, button)))

        if button.find(BACKWARD_LABEL) != -1:  # [<<2]
            label = f'{BACKWARD_LABEL}'
        elif button.find(FORWARD_LABEL) != -1:  # [7>>]
            label = f'{FORWARD_LABEL}'
        else:  # [4]
            label = f'{button} {page_label}'

        menu.append(
            InlineKeyboardButton(
                text=label,
                callback_data=f'{str(page_number)}'
                              f'{str(callback_signature_char)}'
            )
        )

    return [menu]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paginator', 'paginator.composers']

package_data = \
{'': ['*']}

install_requires = \
['python-telegram-bot>=13.8.1,<14.0.0']

setup_kwargs = {
    'name': 'ptb-menu-pagination',
    'version': '0.2.1',
    'description': 'Creates google-like navigation menu using python-telegram-bot wrapper',
    'long_description': '# python telegram bot menu pagination\n![Actions Status](https://github.com/SergSm/ptb-menu-pagination/workflows/ci/badge.svg)\n[![Maintainability](https://api.codeclimate.com/v1/badges/9eade003d09d837c852e/maintainability)](https://codeclimate.com/github/SergSm/ptb-menu-pagination/maintainability)\n[![Test Coverage](https://api.codeclimate.com/v1/badges/9eade003d09d837c852e/test_coverage)](https://codeclimate.com/github/SergSm/ptb-menu-pagination/test_coverage)\n\n# Description\n\nMakes a google style pagination line for a list of items.\n\n![](https://github.com/SergSm/ptb-menu-pagination/blob/main/example/media/example2.png)\n\nIn other words it builds a menu for navigation if you have \na lot of search results or whatever list of anything \n\n![](https://github.com/SergSm/ptb-menu-pagination/blob/main/example/media/example3.png)\n\n\n### Installation\n\n```\npip install ptb-menu-navigation\n```\n\n\nor if you are working with source code and use Poetry tool:\n\n```\nmake install\n```\n\n### Usage\n```python\nfrom paginator import get_menu\n```\n\n\nUse ```get_menu``` function to create a line of pages\n\n#### Example:\n```python\nfrom paginator import get_menu \nfrom dataclasses import dataclass\n\n# Define initial menu settings in the dataclass.\n@dataclass\nclass Menu:\n    items_per_page: int = 10\n    pages_per_line: int = 3\n    navigation_signature: str = \'±\'\n    page_label: str = \' p. \'\n\n# Add the initial call of get_menu\ndef handling_input(update, context):\n    # ...\n    # On first invocation\n    navigation = get_menu(total_items=len(search_results),\n                          current_page=1,\n                          menu_settings=Menu)\n    # ...\n\n# Add a callback to handle a page switching  \ndef navigate(update, context):\n    # ...\n    navigation = get_menu(total_items=len(search_results),\n                          current_page=int(current_page),\n                          menu_settings=Menu)     \n    # ...            \n```\nwhere ```search_results``` is a list of strings and ```current_page```\nis a number extracted from a ```callback_data```.\n\nSee ```examples/search_bot.py```\n\n### Demo bot launch\nCreate a ```.env``` file with a ```TOKEN``` variable\ninside of an ```/examples``` for launching \nthe\n[demo](https://github.com/SergSm/ptb-menu-pagination/blob/main/example/search_bot.py) bot.\\\neg:\\\n```TOKEN=<YOUR_TELEGRAM_BOT_TOKEN_FROM_BOT_FATHER>```\n\nYou may also provide some additional menu values in the same ```.env``` file:\n```\nITEMS_PER_PAGE=1\nPAGES_PER_LINE=1\nNAVIGATION_SIGNATURE="±"\nPAGE_LABEL=" p. "\n```\n',
    'author': 'Smirnov Sergey',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/SergSm/ptb-menu-pagination',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from math import ceil
from dataclasses import dataclass

from telegram import InlineKeyboardButton

from paginator.composers.map import build
from paginator.composers.menu import compose_tg_menu


# Navigation
@dataclass
class Menu:
    items_per_page: int
    pages_per_line: int
    navigation_signature: str
    page_label: str


def get_menu(total_items: int,
             current_page: int,
             menu_settings: Menu) -> list[list[InlineKeyboardButton]]:

    total_pages = ceil(total_items / menu_settings.items_per_page)
    total_lines = ceil(total_pages / menu_settings.pages_per_line)

    # eg:  ['<<2', '4', '5V', '6', '7>>']
    navigation_mockup = build(
        total_pages=total_pages,
        total_lines=total_lines,
        current_page=current_page,
        pages_per_line=menu_settings.pages_per_line
    )

    navigation_menu = compose_tg_menu(
        navigation_mockup,
        menu_settings.navigation_signature,
        menu_settings.page_label
    )

    return navigation_menu
